/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.desktop;

import com.vektorsoft.demux.core.app.DMXApplication;
import com.vektorsoft.demux.desktop.app.DMXDesktopApplication;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Filter;
import org.osgi.util.tracker.ServiceTracker;

/**
 * Activator for desktop bundle.
 *
 * @author Vladimir Djurovic
 */
public class DMXDesktopActivator implements BundleActivator {

    /**
     * Tracker for JavaFX window service.
     */
    private ServiceTracker windowServiceTracker;
    
    /**
     * This method will start the bundle and register basic services.
     * 
     * @param context bundle context
     * @throws Exception if an error occurs
     */
    @Override
    public void start(BundleContext context) throws Exception {
        
        DMXApplication application = new DMXDesktopApplication();
        context.registerService(DMXApplication.class, application, null);
           
        String filterString = "(objectClass=com.vektorsoft.demux.desktop.jfx.main.svc.JFXWindowService)";
        Filter filter = context.createFilter(filterString);
        windowServiceTracker = new ServiceTracker(context, filter, new WindowServiceCustomizer(context));
        windowServiceTracker.open();
    }

    @Override
    public void stop(BundleContext context) throws Exception {
        windowServiceTracker.close();
    }

}
