/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.desktop.svc;

import com.vektorsoft.demux.core.mva.DMXAdapter;
import com.vektorsoft.demux.core.mva.DMXController;
import com.vektorsoft.demux.core.mva.DMXModel;
import com.vektorsoft.demux.core.mva.ExecutionException;
import com.vektorsoft.demux.core.mva.NoSuchControllerException;
import java.util.Map;
import java.util.Set;


/**
 * Implementation of {@link DMXAdapter} used in desktop applications. This adapter can be used with both GUI and 
 * non-GUI (console) applications, depending on {@link DMXViewManager} provided to it.
 *
 * @author Vladimir Djurovic
 */
public class DesktopAdapter extends DMXAdapter{
    
    /**
     * Creates new instance.
     */
    public DesktopAdapter(){
        super();
        setModel(new DMXModel());
    }

    /**
     * Invokes controller with specified ID. This method is called whenever controller needs to be executed. It will
     * call controllers <code>execute()</code> method with arguments provided.
     * 
     * @param controllerId controller ID
     * @param args execution arguments
     */
    public void invokeController(String controllerId, Object... args) {
        try {
            DMXController controller = controllerMap.get(controllerId);
            if(controller == null){
                throw new NoSuchControllerException("Controller not found: " + controllerId);
            }
            Set<String> names = controller.getControllerDataNames();
            Map<String, Object> dataMap = dataModel.getDataValues(names.toArray(new String[names.size()]));
            
            controller.updateFromModel(dataMap);
            controller.execute(args);
            dataModel.setDataValues(controller.updateModel());
            
        } catch (ExecutionException | NoSuchControllerException ex) {
            throw new RuntimeException(ex);
        }
    }
    
}
